import java.util.*;
import java.io.*;

/**
 * A KernelMatrix
 * @copyright Renqiang Min/Hojjat Ghaderi 
 * */
public class KernelMatrix {
  
  private ArrayList<String> seqList; //the list of sequences
  private int dim; //dimension of the kernel matrix
  private KMerTrie[] trieList; //array of tries corresponding to the sequences
  private Alphabet alphabet; //the alphabet shared by sequences
  private int k;    //size of kmer
  private int m;    //mismatch number, 0 mean exact match
  private int[][] matrix;  //the matrix 
  
  /**
   * Initilizes a KernelMatrix by reading sequences from the given file.
   * @param filename Name of the file containing sequences
   * @param alphabet The alphabet shared by sequences
   * @param k size of kmer
   * @param m mismatch number, 0 mean exact match
   * @throws IOException if file has problem
   * throws TrieException if cannot create Tries for the sequences
   */
  public KernelMatrix(String filename, Alphabet alphabet, int k, int m) 
    throws IOException, TrieException {   
    seqList = new ArrayList<String>();
    try {
      BufferedReader buf = new BufferedReader(new FileReader(filename));
      String line = buf.readLine();
      if (line == null) {
        IOException ioe = new IOException("Sequence list file is empty!");
        throw ioe;
      }
      while (line != null) { 
        seqList.add(line);
        line = buf.readLine();
      }
      seqList.trimToSize();
    } catch (IOException ie) {
      IOException ioe = new IOException("cannot read from Sequence file.");
      throw ioe;
    }
    initialize(seqList, alphabet, k, m); 
  }
  
  /**
   * Initializes a KernelMatrix using the given sequece list.
   * @param seqList The list of sequences
   * @param alphabet The alphabet shared by sequences
   * @param k size of kmer
   * @param m mismatch number, 0 mean exact match
   * throws TrieException if cannot create Tries for the sequences
   */ 
  public KernelMatrix(ArrayList<String> seqList, Alphabet alphabet, int k, 
                      int m) throws TrieException {
    initialize(seqList, alphabet, k, m); 
  }

   
  /**
   * Initializes the KernelMatrix using the given sequece list and creates
   * tries corresponding to the sequences.
   * @param seqList The list of sequences
   * @param alphabet The alphabet shared by sequences
   * @param k size of kmer
   * @param m mismatch number, 0 mean exact match
   * throws TrieException if cannot create Tries for the sequences
   */ 
  private void initialize(ArrayList<String> seqList, Alphabet alphabet, 
                          int k, int m) throws TrieException {
    this.seqList = seqList;
    this.k = k;
    this.m = m;
    this.alphabet = alphabet;
    this.dim = seqList.size();
    trieList = new KMerTrie[dim];
    matrix = new int[dim][dim];
    
    for (int i = 0; i < dim; i ++) {
       trieList[i] = new KMerTrie(alphabet, k, m, seqList.get(i));       
    }
  }

  /**  
   * creating a string out of the matrix
   * @return a string representation of the matrix
   */
  public String toString() {
    String s = "";
    if (matrix != null) {
      for (int i = 0; i < dim; i ++) {
        for (int j = 0; j < dim; j ++) {
          if (j != 0) {
	    s = s + "\t"; 
	  }
          s = s + Integer.toString(matrix[i][j]);
        }
        s = s + "\n";
      }
    }
    return s;
  }

  

  /**
   * calculate and set the elements of the kernel matrix. See the handout.
   * @return the matrix in the Kernel matrix.
   * throws TrieException if triList is null.
   */ 
  public int[][] calcKernelMatrix() throws TrieException {
    //TO DO: COMPLETE THIS METHOD. YOUR CODE MUST BE EFFICIENT.
    //       (NOTE THAT THE MATRIX IS SYMETRIC!)
    //       
    //       Hint: for each row, get the list of kmers corresponding
    //             to the sequence, then use the dotproduct to compute
    //             each element in the row

  }
    
  
}
